@@ -18,6 +18,8 @@ import urwid_timed_progress
 from . import urwid_multi_key_support
 
 QUIT = 'quit'
+QUIT_ASK = 'quit --ask'
+QUIT_ASK_IF_LONG = 'quit --ask-if-long-startup'
 CONFIG = 'config'
 HELP_LIST_OF_KEY_MAPPINGS = 'help key-mappings'
 HELP_LIST_OF_COMMANDS = 'help commands'
@@ -32,7 +34,7 @@ urwid.command_map['f1'] = HELP_LIST_OF_KEY_MAPPINGS
 urwid.command_map['f2'] = HELP_LIST_OF_KEY_MAPPINGS
 urwid.command_map['f3'] = HELP_LIST_OF_COMMANDS
 urwid.command_map['f4'] = HELP_CONFIG
-urwid.command_map['q'] = QUIT
+urwid.command_map['q'] = QUIT_ASK_IF_LONG
 
 
 from . import urwid_dialog
@@ -67,6 +69,8 @@ urwid.command_map.implemented_commands = {
 	urwid.ACTIVATE,
 	CANCEL,
 	QUIT,
+	QUIT_ASK,
+	QUIT_ASK_IF_LONG,
 	CONFIG,
 	HELP_LIST_OF_KEY_MAPPINGS,
 	HELP_LIST_OF_COMMANDS,
@@ -124,6 +128,24 @@ def time_difference_to_str(seconds: float) -> str:
 	return pattern_hours.value.format(s=s, m=m, h=h)
 
 
+View = urwid_dialog.View
+
+class LastView(View):
+
+	def __init__(self,
+		widget: urwid.Widget,
+		help_bar: urwid_multi_key_support.HelpBar,
+	) -> None:
+		self.widget = widget
+		self.help_bar = help_bar
+
+	def get_box_widget(self) -> urwid.Widget:
+		return self.widget
+
+	def get_help_bar(self) -> urwid_multi_key_support.HelpBar:
+		return self.help_bar
+
+
 class App(urwid_directory_chooser.App):
 
 	resource_help_config = 'doc/config.md'
@@ -145,6 +167,7 @@ class App(urwid_directory_chooser.App):
 
 	progress_bar_update_time = Config('progress-bar.update-time', 0.5, help='number of seconds between two updates of the progress bar')
 	time_before_opening_loading_screen = Config('time-before-opening-progress-bar', 1, help='wait time before opening the progress bar to avoid an unnecessary progress bar when scanning small directories')
+	time_long_startup = Config('time-long-startup', 5.0, help='if scanning the directories for changes has taken longer than this number of seconds it is considered long and `quit --ask-if-long-startup` will ask before quitting')
 
 	WC_PATH = '{path}'
 	cmd_file_browser = Config('cmd.filebrowser', CommandWithAlternatives([
@@ -153,19 +176,16 @@ class App(urwid_directory_chooser.App):
 		['vim', WC_PATH],
 	]), help='the command used to open a directory')
 
-	help_bar_content_progress_bar = [
-		urwid_multi_key_support.HelpItem(QUIT, 'quit'),
-	]
-
 	# ------- app.init -------
 
 	def __init__(self, path_src: typing.Optional[str], path_dst: typing.Optional[str]) -> None:
 		self.delay_show()
+		self._body_widget = None
 		lsblk.udisksctl.set_logger(self)
 		self.frameview = urwid.Frame(None)
 		self.screen = self.create_screen()
 		self.loop = urwid.MainLoop(self.frameview, screen=self.screen, input_filter=self.input_filter, unhandled_input=self.unhandled_input, handle_mouse=self.handle_mouse)
-		self._last_widget: typing.Optional[urwid.Widget] = None
+		self._last_view: typing.Optional[LastView] = None
 		urwid_colors.ColorStr.set_logger(self)
 		urwid_colors.ColorStr.set_register_color(self.register_color)
 		urwid_colors.ColorConfig.set_register_color(self.register_color)
@@ -247,8 +267,7 @@ class App(urwid_directory_chooser.App):
 			self.counter_thread = threading.Thread(target=self._count_number_files_in_other_thread, args=[paths], daemon=True)
 			self.counter_thread.start()
 			self.progressbar = MyProgressBar(self)
-			self.frameview.body = urwid.Filler(self.progressbar)
-			self.show_help_bar(self.help_bar_content_progress_bar, urwid.command_map, edit_context=False)
+			self.open_view(self.progressbar)
 			self.loop.set_alarm_in(self.progress_bar_update_time, self.update_progressbar)
 		else:
 			self.after_load()
@@ -278,21 +297,26 @@ class App(urwid_directory_chooser.App):
 		if self._exception_in_loader:
 			raise self._exception_in_loader
 		root_directory_node = FileOrDirectoryNode(self.meta_node)
-		self.statistics_widget = StatisticsWidget()
 		treeview = DiffWidget(self, urwid.TreeWalker(root_directory_node))
-		treeview.set_error_listener(self.on_error)
-		widget = urwid.Frame(treeview, footer=self.statistics_widget)
-		self.frameview.body = widget
+		self.open_view(treeview)
 		self.show_info('scanning the directories has taken %s' % time_difference_to_str(self.t1-self.t0))
 
-	def show_help_bar(self, help_bar_content: typing.Sequence[urwid_multi_key_support.HelpItem], command_map: urwid_multi_key_support.SubCommandMap, *, edit_context: bool) -> None:
-		self.frameview.header = urwid_multi_key_support.HelpBar(help_bar_content, command_map, self, edit_context=edit_context)
+	def open_view(self, view: View) -> None:
+		self.frameview.body = view.get_box_widget()
+		if self._body_widget is not None:
+			self._body_widget = self.frameview.body
+		self.show_help_bar(view.get_help_bar())
+		self.is_ask_to_quit_dialog_open = False
 
-	def save_help_bar(self) -> None:
-		self._prev_help_bar = self.frameview.header
+	def show_help_bar(self, help_bar: urwid_multi_key_support.HelpBar) -> None:
+		self.frameview.header = help_bar
 
-	def restore_help_bar(self) -> None:
-		self.frameview.header = self._prev_help_bar
+	def save_view(self) -> LastView:
+		if self._body_widget is not None:
+			main_widget = self._body_widget
+		else:
+			main_widget = self.frameview.body
+		return LastView(main_widget, help_bar=self.frameview.header)
 
 
 	def load_backup_plan(self, name: str) -> bool:
@@ -340,10 +364,10 @@ class App(urwid_directory_chooser.App):
 	def ask_to_create_backup_plan(self, name: str, fn: str) -> None:
 		dialog = urwid_dialog.YesNoDialog(self, f'Backup plan {name!r} does not exist. Do you want to create it?', yes=lambda: self.ask_for_backup_plan_data(name, fn), no=self.quit)
 		dialog.set_key_handler(self.handle_key)
-		self.frameview.body = dialog
+		self.open_view(dialog)
 
 	def ask_for_backup_plan_data(self, name: str, fn: str) -> None:
-		self.frameview.body = BackupPlanWidget(self, name, fn, create=self.create_backup_plan, cancel=self.quit)
+		self.open_view(BackupPlanWidget(self, name, fn, create=self.create_backup_plan, cancel=self.quit))
 
 	def create_backup_plan(self, widget: 'BackupPlanWidget') -> None:
 		name = widget.get_name()
@@ -357,12 +381,12 @@ class App(urwid_directory_chooser.App):
 		self.ask_to_edit_backup_plan(fn)
 
 	def ask_to_edit_backup_plan(self, fn: str) -> None:
-		self.frameview.body = urwid_dialog.YesNoDialog(
+		self.open_view(urwid_dialog.YesNoDialog(
 			self,
 			'I have created {fn}.\nDo you want to edit it (e.g. to change default directions)?'.format(fn=fn),
 			yes = lambda: self.edit_backup_plan_and_start(fn),
 			no = self.init_model,
-		)
+		))
 
 	def edit_backup_plan_and_start(self, fn: str) -> None:
 		self.run_tui_command([self.get_editor(), fn])
@@ -452,8 +476,8 @@ class App(urwid_directory_chooser.App):
 		sys.exit(exit_code)
 
 
-	def handle_key(self, size: URWID_TYPE_SIZE, key: URWID_TYPE_KEY) -> typing.Optional[URWID_TYPE_KEY]:
-		cmd = urwid.command_map[key]
+	def handle_key(self, command_map: urwid_multi_key_support.SubCommandMap, size: URWID_TYPE_SIZE, key: URWID_TYPE_KEY) -> typing.Optional[URWID_TYPE_KEY]:
+		cmd = command_map[key]
 		if cmd == HELP_LIST_OF_KEY_MAPPINGS:
 			self.open_help_list_of_key_mappings()
 		elif cmd == HELP_LIST_OF_COMMANDS:
@@ -465,6 +489,10 @@ class App(urwid_directory_chooser.App):
 			self.open_config()
 		elif cmd == QUIT:
 			self.quit()
+		elif cmd == QUIT_ASK:
+			self.ask_to_quit()
+		elif cmd == QUIT_ASK_IF_LONG:
+			self.ask_to_quit_if_long_startup()
 		else:
 			return key
 
@@ -488,9 +516,36 @@ class App(urwid_directory_chooser.App):
 		cmd = urwid.command_map[key]
 		if cmd == QUIT:
 			self.quit()
+		elif cmd == QUIT_ASK:
+			self.ask_to_quit()
+		elif cmd == QUIT_ASK_IF_LONG:
+			self.ask_to_quit_if_long_startup()
 		else:
 			return False
-		return True  # type: ignore [unreachable]
+		return True
+
+	def ask_to_quit_if_long_startup(self) -> None:
+		now = time.time()
+		t0 = getattr(self, 't0', now)
+		t1 = getattr(self, 't1', now)
+		dt = t1 - t0
+		if dt > self.time_long_startup:
+			self.ask_to_quit()
+		else:
+			self.quit()
+
+	def ask_to_quit(self) -> None:
+		if self.is_ask_to_quit_dialog_open:
+			return
+
+		last_view = self.save_view()
+		self.open_view(urwid_dialog.YesNoDialog(
+			self,
+			'Do you want to close this program?'.format(),
+			yes = self.quit,
+			no = lambda: self.open_view(last_view)
+		))
+		self.is_ask_to_quit_dialog_open = True
 
 	def quit(self) -> typing.NoReturn:
 		raise urwid.ExitMainLoop()
@@ -558,15 +613,13 @@ class App(urwid_directory_chooser.App):
 		self.open_help(HelpWidgetFromResource(self, self.resource_help_config))
 
 	def open_help(self, widget: urwid.Widget) -> None:
-		if self._last_widget is None:
-			self.save_help_bar()
-			self._last_widget = self.frameview.body
-		self.frameview.body = widget
-		self.show_help_bar(widget.help_bar_content, widget._command_map, edit_context=False)
+		if self._last_view is None:
+			self._last_view = self.save_view()
+		self.open_view(widget)
 
 	def close_help(self) -> None:
-		self.frameview.body = self._last_widget
-		self.restore_help_bar()
+		assert self._last_view is not None
+		self.open_view(self._last_view)
 
 
 	# ------- main -------
@@ -588,7 +641,7 @@ class App(urwid_directory_chooser.App):
 
 # ========== main widget ==========
 
-class DiffWidget(urwid.TreeListBox, urwid_multi_key_support.MultiKeySupport):
+class DiffWidget(urwid.TreeListBox, urwid_multi_key_support.MultiKeySupport, View):
 
 	help_bar_content = Config('diff.help-bar', [
 		urwid_multi_key_support.HelpItem(HELP_LIST_OF_KEY_MAPPINGS, 'help'),
@@ -650,6 +703,8 @@ class DiffWidget(urwid.TreeListBox, urwid_multi_key_support.MultiKeySupport):
 		HELP_CONFIG,
 		CONFIG,
 		QUIT,
+		QUIT_ASK,
+		QUIT_ASK_IF_LONG,
 	}
 
 	def __init__(self, app: App, tree_walker: urwid.TreeWalker) -> None:
@@ -657,14 +712,23 @@ class DiffWidget(urwid.TreeListBox, urwid_multi_key_support.MultiKeySupport):
 		self.init_multi_key_support(app)
 		self.app = app
 		self.clipboard = clipboard.Clipboard(self.app)
+		self.statistics_widget = StatisticsWidget()
 		self.update_statistics()
-		self.app.show_help_bar(self.help_bar_content, self._command_map, edit_context=False)
 
 	def update_statistics(self) -> None:
-		self.app.statistics_widget.set(self.focus.get_model())
+		self.statistics_widget.set(self.focus.get_model())
 
 
-	# ------- overriding methods -------
+	# ------- implementing View methods -------
+
+	def get_help_bar(self) -> urwid_multi_key_support.HelpBar:
+		return urwid_multi_key_support.HelpBar(self.help_bar_content, self._command_map, self.app, edit_context=False)
+
+	def get_box_widget(self) -> urwid.Widget:
+		return urwid.Frame(self, footer=self.statistics_widget)
+
+
+	# ------- overriding widget methods -------
 
 	def keypress(self, size: URWID_TYPE_SIZE, key: URWID_TYPE_KEY) -> typing.Optional[URWID_TYPE_KEY]:
 		if self.waiting_for_next_key(key):
@@ -687,22 +751,22 @@ class DiffWidget(urwid.TreeListBox, urwid_multi_key_support.MultiKeySupport):
 				try:
 					self.focus.get_model().toggle_direction()
 				except model.CommandNotAllowed as e:
-					self.notify_error(e)
+					self.app.show_error(e)
 			elif func == FUNC_NODE_TOGGLE_IGNORE:
 				try:
 					self.focus.get_model().toggle_ignore()
 				except model.CommandNotAllowed as e:
-					self.notify_error(e)
+					self.show_error(e)
 			elif func == FUNC_NODE_SET_DIRECTION_SRC_TO_DST:
 				try:
 					self.focus.get_model().set_direction_recursively(model.DIRECTION.SRC_TO_DST)
 				except model.CommandNotAllowed as e:
-					self.notify_error(e)
+					self.show_error(e)
 			elif func == FUNC_NODE_SET_DIRECTION_DST_TO_SRC:
 				try:
 					self.focus.get_model().set_direction_recursively(model.DIRECTION.DST_TO_SRC)
 				except model.CommandNotAllowed as e:
-					self.notify_error(e)
+					self.show_error(e)
 			elif func == FUNC_NODE_UPDATE:
 				self.focus.get_model().update()
 				self.focus.update_widget()
@@ -722,7 +786,7 @@ class DiffWidget(urwid.TreeListBox, urwid_multi_key_support.MultiKeySupport):
 				cmd = self.cmd_diff.replace(self.PATH_SRC, cn.path_src).replace(self.PATH_DST, cn.path_dst)
 				self.app.run_tui_command(cmd)
 
-			elif self.app.handle_key(size, key) is None:
+			elif self.app.handle_key(self._command_map, size, key) is None:
 				pass
 
 			else:
@@ -769,14 +833,6 @@ class DiffWidget(urwid.TreeListBox, urwid_multi_key_support.MultiKeySupport):
 		parent_tree_node.last_focus = tree_node
 
 
-	def set_error_listener(self, callback: typing.Callable[[Exception], None]) -> None:
-		self.error_listener = callback
-
-	def notify_error(self, exception: Exception) -> None:
-		if hasattr(self, 'error_listener'):
-			self.error_listener(exception)
-
-
 class ComparisonWidget(urwid.TreeWidget):
 
 	indent_cols: int  # inherited from TreeWidget
@@ -1144,8 +1200,8 @@ class PathsGroup(urwid.WidgetWrap):
 		return self._w.base_widget
 
 	def __init__(self, app: App, master: 'BackupPlanWidget') -> None:
-		self.path_src = urwid_directory_chooser.PathEdit(app, 'src: ', '', master)
-		self.path_dst = urwid_directory_chooser.PathEdit(app, 'dst: ', '', master)
+		self.path_src = urwid_directory_chooser.PathEdit(app, 'src: ', '')
+		self.path_dst = urwid_directory_chooser.PathEdit(app, 'dst: ', '')
 		self.btn_add = urwid_dialog.ColorButton('+', master.add_path_group)
 		self.btn_del = urwid_dialog.ColorButton('-', master.del_path_group)
 
@@ -1196,7 +1252,7 @@ class PathsGroup(urwid.WidgetWrap):
 		return self.path_dst.get_path()
 
 
-class BackupPlanWidget(urwid.WidgetWrap):
+class BackupPlanWidget(urwid.WidgetWrap, View):
 
 	help_bar_content = Config('backup-plan-widget.help-bar.button', [
 		urwid_multi_key_support.HelpItem(urwid.ACTIVATE, 'click button'),
@@ -1220,8 +1276,18 @@ class BackupPlanWidget(urwid.WidgetWrap):
 		self.pile = urwid_dialog.TabAwarePile([text, sep, PathsGroup(app, self), self.buttons_frame])
 		widget = urwid.Filler(self.pile)
 		super().__init__(widget)
+		self.help_bar_edit = urwid_multi_key_support.HelpBar(urwid_directory_chooser.PathEdit.help_bar_content.value, self._command_map, self.app, edit_context=True)
+		self.help_bar_button = urwid_multi_key_support.HelpBar(self.help_bar_content, self._command_map, self.app, edit_context=False)
 		self.on_path_edit_focus_change(self.is_edit_path_focused())
 
+	# ------- View methods -------
+
+	def get_box_widget(self) -> urwid.Widget:
+		return self
+
+	def get_help_bar(self) -> urwid_multi_key_support.HelpBar:
+		return self.help_bar_edit
+
 	# ------- update help bar -------
 
 	def keypress(self, size: URWID_TYPE_SIZE, key: URWID_TYPE_KEY) -> typing.Optional[URWID_TYPE_KEY]:
@@ -1243,12 +1309,10 @@ class BackupPlanWidget(urwid.WidgetWrap):
 	def on_path_edit_focus_change(self, focus: bool) -> None:
 		self.was_edit_path_focused = focus
 		if focus:
-			content = urwid_directory_chooser.PathEdit.help_bar_content.value
-			edit_context = True
+			help_bar = self.help_bar_edit
 		else:
-			content = self.help_bar_content
-			edit_context = False
-		self.app.show_help_bar(content, self._command_map, edit_context=edit_context)
+			help_bar = self.help_bar_button
+		self.app.show_help_bar(help_bar)
 
 	# ------- buttons -------
 
@@ -1291,13 +1355,17 @@ class BackupPlanWidget(urwid.WidgetWrap):
 
 # ========== progress bar ==========
 
-class MyProgressBar(urwid_timed_progress.TimedProgressBar, urwid_multi_key_support.MultiKeySupport):
+class MyProgressBar(urwid_timed_progress.TimedProgressBar, urwid_multi_key_support.MultiKeySupport, View):
 
 	color_todo = urwid_colors.ColorConfig('progress-bar.color.todo', 'default')
 	color_done = urwid_colors.ColorConfig('progress-bar.color.done', 'green/white')
 	expected_number_of_files = Config('progress-bar.estimated-number-of-files', 500000, help='a rough guess how many files need to be processed, used for the progress bar before finish counting the real number')
 	unit = 'files'
 
+	help_bar_content = Config('progress-bar.help-bar', [
+		urwid_multi_key_support.HelpItem([QUIT, QUIT_ASK, QUIT_ASK_IF_LONG], 'quit'),
+	])
+
 	def __init__(self, app: App) -> None:
 		super().__init__(self.color_todo, self.color_done, done=self.expected_number_of_files, units=self.unit)
 		self.init_multi_key_support(app)
@@ -1312,13 +1380,21 @@ class MyProgressBar(urwid_timed_progress.TimedProgressBar, urwid_multi_key_suppo
 			self.reset_command_map()
 			return None
 
+		out = self.app.handle_key(self._command_map, size, key)
 		self.reset_command_map()
-		return self.app.handle_key(size, key)
+		return out
+
+	def get_help_bar(self) -> urwid_multi_key_support.HelpBar:
+		return urwid_multi_key_support.HelpBar(self.help_bar_content, self._default_command_map, self.app, edit_context=False)
+
+	def get_box_widget(self) -> urwid.Widget:
+		return urwid.Filler(self)
+
 
 
 # ========== help ==========
 
-class HelpWidget(urwid.WidgetWrap):
+class HelpWidget(urwid.WidgetWrap, View):
 
 	help_bar_content = Config('help-page.help-bar', [
 		urwid_multi_key_support.HelpItem(CANCEL, 'back'),
@@ -1336,17 +1412,20 @@ class HelpWidget(urwid.WidgetWrap):
 		self.app = app
 		widget.set_key_handler(self.handle_key)
 
-	def handle_key(self, size: URWID_TYPE_SIZE, key: URWID_TYPE_KEY) -> typing.Optional[URWID_TYPE_KEY]:
-		cmd = self._command_map[key]
+	def handle_key(self, command_map: urwid_multi_key_support.SubCommandMap, size: URWID_TYPE_SIZE, key: URWID_TYPE_KEY) -> typing.Optional[URWID_TYPE_KEY]:
+		cmd = command_map[key]
 		if cmd == CANCEL:
 			self.app.close_help()
-		elif self.app.handle_key(size, key) is None:
+		elif self.app.handle_key(command_map, size, key) is None:
 			pass
 		else:
 			return key
 
 		return None
 
+	def get_box_widget(self) -> urwid.Widget:
+		return self
+
 
 class HelpWidgetListOfKeyMappings(HelpWidget):
 
@@ -1357,8 +1436,12 @@ class HelpWidgetListOfKeyMappings(HelpWidget):
 			widget = urwid_multi_key_support.PressedKeysWidget(app, pressed_keys=[], command_map=command_map, title=title)
 			widgets.extend(widget.body)
 		widget = urwid_multi_key_support.ExtendedListBox(app, widgets)
+		self.listbox = widget
 		super().__init__(widget, app)
 
+	def get_help_bar(self) -> urwid_multi_key_support.HelpBar:
+		return urwid_multi_key_support.HelpBar(self.help_bar_content, self.listbox._default_command_map, self.app, edit_context=False)
+
 
 class HelpWidgetListOfImplementedCommands(HelpWidget):
 
@@ -1370,8 +1453,12 @@ class HelpWidgetListOfImplementedCommands(HelpWidget):
 			for cmd in sorted(cmdmap.implemented_commands):
 				widgets.append(HelpCommandWidget(cmd))
 		widget = urwid_multi_key_support.ExtendedListBox(app, widgets)
+		self.listbox = widget
 		super().__init__(widget, app)
 
+	def get_help_bar(self) -> urwid_multi_key_support.HelpBar:
+		return urwid_multi_key_support.HelpBar(self.help_bar_content, self.listbox._default_command_map, self.app, edit_context=False)
+
 class HelpWidgetFromResource(HelpWidget):
 
 	PREFIX_SECTION = '# '
@@ -1392,6 +1479,7 @@ class HelpWidgetFromResource(HelpWidget):
 		assert raw is not None
 		widgets = self.parse(raw.decode('utf-8'))
 		widget = urwid_multi_key_support.ExtendedListBox(app, widgets)
+		self.listbox = widget
 		super().__init__(widget, app)
 
 	def parse(self, raw: str) -> typing.Sequence[urwid.Widget]:
@@ -1437,6 +1525,9 @@ class HelpWidgetFromResource(HelpWidget):
 			markup.append(text)
 		return HelpLineWidget(markup)
 
+	def get_help_bar(self) -> urwid_multi_key_support.HelpBar:
+		return urwid_multi_key_support.HelpBar(self.help_bar_content, self.listbox._default_command_map, self.app, edit_context=False)
+
 
 class HelpLineWidget(urwid.Text):
 	pass
