@@ -38,6 +38,8 @@ urwid.command_map['G'] = CURSOR_MAX_DOWN
 T = typing.TypeVar('T')
 
 
+View = urwid_dialog.View
+
 class App(urwid_multi_key_support.OverlayPressedKeysOpener):
 
 	@abc.abstractmethod
@@ -45,7 +47,15 @@ class App(urwid_multi_key_support.OverlayPressedKeysOpener):
 		raise NotImplementedError()
 
 	@abc.abstractmethod
-	def show_help_bar(self, help_bar_content: typing.Sequence[HelpItem], command_map: urwid_multi_key_support.SubCommandMap, *, edit_context: bool) -> None:
+	def open_view(self, view: View) -> None:
+		raise NotImplementedError()
+
+	@abc.abstractmethod
+	def show_help_bar(self, help_bar: urwid_multi_key_support.HelpBar) -> None:
+		raise NotImplementedError()
+
+	@abc.abstractmethod
+	def save_view(self) -> View:
 		raise NotImplementedError()
 
 
@@ -120,7 +130,7 @@ class PseudoDevice(Device):
 	def unmount(self) -> typing.NoReturn:
 		raise SubprocessException('%r is not a device to be unmounted' % self.mountpoint)
 
-class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport):
+class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport, View):
 
 	BTN_SELECT = 'select'
 	BTN_CANCEL = 'cancel'
@@ -135,11 +145,16 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 
 	help_bar_content = Config('directory-chooser.help-bar', [
 		HelpItem(urwid_dialog.NEXT_SELECTABLE, 'select buttons'),
+		HelpItem([urwid.ACTIVATE, urwid.CURSOR_RIGHT], 'open'),
+		HelpItem(urwid.CURSOR_LEFT, 'back'),
+		HelpItem(RELOAD, 'reload'),
 	])
 	help_bar_content_menu = Config('directory-chooser.help-bar.devices-list', [
 		HelpItem(urwid_dialog.NEXT_SELECTABLE, 'select buttons'),
 		HelpItem(MOUNT, 'mount'),
 		HelpItem(UNMOUNT, 'unmount'),
+		HelpItem([urwid.ACTIVATE, urwid.CURSOR_RIGHT], 'open'),
+		HelpItem(RELOAD, 'reload'),
 	])
 
 	# ------- init -------
@@ -158,6 +173,9 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 		self.listbox.keypress = self.keypress_listbox
 		self.text_path.keypress = self.keypress_path  # type: ignore [assignment]  # This is a bug in mypy https://github.com/python/mypy/issues/2427
 
+		self.help_bar = urwid_multi_key_support.HelpBar(self.help_bar_content, MenuWidget._command_map, self.app, edit_context=False)
+		self.help_bar_menu = urwid_multi_key_support.HelpBar(self.help_bar_content_menu, MenuWidget._command_map, self.app, edit_context=False)
+
 		btn_select = urwid_dialog.ColorButton(self.BTN_SELECT, lambda btn: select(self.text_path.text))
 		btn_cancel = urwid_dialog.ColorButton(self.BTN_CANCEL, lambda btn: cancel())
 
@@ -179,9 +197,12 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 			widget = self.frame
 		super().__init__(widget)
 		self.init_multi_key_support(app)
-		self.update_help_bar(self.is_device_focused())
+		self.was_device_focused = self.is_device_focused()
 
 	def init_path(self, start_path: str) -> None:
+		self.path_selected_in_menu = ''
+		self.path_selected_in_menu_len = 0
+		self.mountpoint_for_path_selected_in_menu = ''
 		if start_path:
 			try:
 				dev = mounter.mount_path_if_necessary(start_path, self.app.password_screen())
@@ -264,12 +285,15 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 			return list(self.iter_menu_widgets(path))
 
 		# I need to check that because user can enter any path in the Edit widget
+		uuid_based_path = path
 		if path.startswith(self.path_selected_in_menu):
-			uuid_based_path = path
 			path = self.mountpoint_for_path_selected_in_menu + path[self.path_selected_in_menu_len:]
 
 		if os.path.isdir(path):
-			directory_names = [fn for fn in os.listdir(path) if os.path.isdir(os.path.join(path, fn))]
+			try:
+				directory_names = [fn for fn in os.listdir(path) if os.path.isdir(os.path.join(path, fn))]
+			except PermissionError as e:
+				return [ErrorWidget(self, uuid_based_path, e)]
 			if not self.show_hidden:
 				directory_names = [dn for dn in directory_names if not self.is_hidden(dn)]
 			directory_names.sort()
@@ -318,9 +342,9 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 
 	def update_help_bar(self, is_device_focused: bool) -> None:
 		if is_device_focused:
-			self.app.show_help_bar(self.help_bar_content_menu, self._command_map, edit_context=False)
+			self.app.show_help_bar(self.help_bar_menu)
 		else:
-			self.app.show_help_bar(self.help_bar_content, self._command_map, edit_context=False)
+			self.app.show_help_bar(self.help_bar)
 		self.was_device_focused = is_device_focused
 
 	def keypress(self, size: URWID_TYPE_SIZE, key: URWID_TYPE_KEY) -> typing.Optional[URWID_TYPE_KEY]:
@@ -375,6 +399,41 @@ class DirectoryChooser(urwid.WidgetWrap, urwid_multi_key_support.MultiKeySupport
 		else:
 			self.frame.focus_position = 'footer'
 
+	# ------- View methods -------
+
+	def get_box_widget(self) -> urwid.Widget:
+		return self
+
+	def get_help_bar(self) -> urwid_multi_key_support.HelpBar:
+		if self.is_device_focused():
+			return self.help_bar_menu
+		return self.help_bar
+
+
+class ErrorWidget(urwid.WidgetWrap):
+
+	color = ColorConfig('directory-chooser.color.error', 'red', 'red')
+	color_focus = color.focus
+
+	def __init__(self, directory_chooser: DirectoryChooser, path: str, e: Exception) -> None:
+		msg = str(e)
+		self.directory_chooser = directory_chooser
+		self.path = path
+		self.name = ''
+		widget = urwid.Text(msg)
+		widget = urwid.AttrMap(widget, self.color, self.color_focus)
+		super().__init__(widget)
+
+	def selectable(self) -> bool:
+		return True
+
+	def keypress(self, size: URWID_TYPE_SIZE, key: URWID_TYPE_KEY) -> typing.Optional[URWID_TYPE_KEY]:
+		cmd = self._command_map[key]
+		if cmd == urwid.ACTIVATE or cmd == urwid.CURSOR_LEFT:
+			self.directory_chooser.go(self.path, os.path.pardir)
+		else:
+			return key
+		return None
 
 class DirectoryWidget(urwid.WidgetWrap):
 
@@ -503,27 +562,26 @@ class PathEdit(urwid.WidgetWrap):
 		urwid_multi_key_support.HelpItem(urwid_dialog.PREV_SELECTABLE, 'focus previous'),
 	])
 
-	def __init__(self, app: App, label: str, start_path: str, master: urwid.WidgetWrap) -> None:
+	def __init__(self, app: App, label: str, start_path: str) -> None:
 		self.app = app
 		self.edit = ExtendedEdit(label, start_path)
-		self.master = master
 		self.has_opened_directory_chooser = False
 		super().__init__(self.edit)
 
 	def on_browse(self) -> None:
 		self.has_opened_directory_chooser = True
-		self._main_widget = self.master._w
-		self.master._w = DirectoryChooser(self.app, self.on_select, self.on_cancel, start_path=self.get_path())
+		self.last_view = self.app.save_view()
+		self.app.open_view(DirectoryChooser(self.app, self.on_select, self.on_cancel, start_path=self.get_path()))
 
 	def on_select(self, path: str) -> None:
 		self.has_opened_directory_chooser = False
 		self.edit.set_edit_text(path)
 		self.edit.set_edit_pos(len(path))
-		self.master._w = self._main_widget
+		self.app.open_view(self.last_view)
 
 	def on_cancel(self) -> None:
 		self.has_opened_directory_chooser = False
-		self.master._w = self._main_widget
+		self.app.open_view(self.last_view)
 
 
 	def validate(self) -> bool:
@@ -577,7 +635,11 @@ if __name__ == '__main__':
 			finally:
 				loop.screen.start()
 
-		def show_help_bar(self, help_bar_content: typing.Sequence[HelpItem], command_map: urwid_multi_key_support.SubCommandMap, *, edit_context: bool) -> None:
+		def save_view(self) -> View:
+			raise NotImplementedError()
+		def open_view(self, view: View) -> None:
+			raise NotImplementedError()
+		def show_help_bar(self, help_bar: urwid_multi_key_support.HelpBar) -> None:
 			pass
 
 		def show_error(self, msg: typing.Union[str, BaseException]) -> None:
