@@ -597,13 +597,13 @@ class ExtendedListBox(urwid.ListBox, MultiKeySupport):
 
 	def __init__(self, pressed_keys_opener: OverlayPressedKeysOpener, widgets: typing.Sequence[urwid.Widget]) -> None:
 		self.ellipsis_widget = urwid.Text((self.color_ellipsis, self.ELLIPSIS), align=urwid.CENTER)
-		self.key_handler: typing.Optional[typing.Callable[[URWID_TYPE_SIZE, URWID_TYPE_KEY], typing.Optional[URWID_TYPE_KEY]]] = None
+		self.key_handler: typing.Optional[typing.Callable[[SubCommandMap, URWID_TYPE_SIZE, URWID_TYPE_KEY], typing.Optional[URWID_TYPE_KEY]]] = None
 
 		body = urwid.SimpleFocusListWalker(widgets)
 		super().__init__(body)
 		self.init_multi_key_support(pressed_keys_opener)
 
-	def set_key_handler(self, callback: typing.Callable[[URWID_TYPE_SIZE, URWID_TYPE_KEY], typing.Optional[URWID_TYPE_KEY]]) -> None:
+	def set_key_handler(self, callback: typing.Callable[[SubCommandMap, URWID_TYPE_SIZE, URWID_TYPE_KEY], typing.Optional[URWID_TYPE_KEY]]) -> None:
 		self.key_handler = callback
 
 
@@ -613,7 +613,7 @@ class ExtendedListBox(urwid.ListBox, MultiKeySupport):
 		if self.waiting_for_next_key(key):
 			return None
 
-		if self.key_handler is not None and self.key_handler(size, key) is None:
+		if self.key_handler is not None and self.key_handler(self._command_map, size, key) is None:
 			self.reset_command_map()
 			return None
 
@@ -745,8 +745,9 @@ class HelpItem:
 	type_article = None
 
 	SEP = ': '
+	CMD_SEP = '||'
 
-	def __init__(self, cmd: str, name: typing.Optional[str] = None) -> None:
+	def __init__(self, cmd: typing.Union[str, typing.Sequence[str]], name: typing.Optional[str] = None) -> None:
 		'''
 		:param cmd: the command for which you want to show the keyboard shortcuts.
 		            Alternatively, if you want to show a group of keyboard shortcuts,
@@ -757,18 +758,29 @@ class HelpItem:
 		:param name: one or two words to describe what cmd does.
 		'''
 		if name is None:
+			assert isinstance(cmd, str)
 			# parsing value from config file
 			name, cmd = cmd.split(self.SEP, 1)
+			cmd = cmd.split(self.CMD_SEP)
 
-		self.cmd = cmd
+		if isinstance(cmd, str):
+			cmd = (cmd,)
+
+		self.cmds = cmd
 		self.name = name
 
+	def format_cmd(self) -> str:
+		return self.CMD_SEP.join(self.cmds)
+
 	def __str__(self) -> str:
-		return self.name + self.SEP + self.cmd
+		return self.name + self.SEP + self.format_cmd()
+
+	def __repr__(self) -> str:
+		return f'{type(self).__name__}({self.cmds!r}, {self.name!r})'
 
 	def __eq__(self, other: typing.Any) -> bool:
 		if isinstance(other, HelpItem):
-			return self.cmd == other.cmd and self.name == other.name
+			return set(self.cmds) == set(other.cmds) and self.name == other.name
 		return NotImplemented
 
 
@@ -803,22 +815,24 @@ class HelpBar(urwid.WidgetWrap):
 		markup: URWID_TYPE_MARKUP = []
 		sep = ColorStr.to_markup(self.sep)
 		for help_item in content:
-			if help_item.cmd.startswith('<'):
-				if not self.is_key_mapped(help_item.cmd, command_map):
+			if len(help_item.cmds) == 1 and help_item.cmds[0].startswith('<'):
+				key = help_item.cmds[0]
+				if not self.is_key_mapped(key, command_map):
 					keys = []
 					if self.unmapped_commands_are_errors:
-						self.logger.show_error('no commands are mapped to the key %s' % help_item.cmd)
+						self.logger.show_error('no commands are mapped to the key %s' % key)
 				else:
-					keys = [KeyMapper.format_keys(KeyMapper.parse_keys(help_item.cmd))]
+					keys = [KeyMapper.format_keys(KeyMapper.parse_keys(key))]
 			else:
-				keys = [key for key, cmd in all_key_mappings if cmd == help_item.cmd and (not edit_context or self.is_key_usable_in_edit_context(key))]
+				keys = [key for key, cmd in all_key_mappings if cmd in help_item.cmds and (not edit_context or self.is_key_usable_in_edit_context(key))]
 				if self.unmapped_commands_are_errors and not keys:
-					self.logger.show_error('no keys are mapped to the command %r' % help_item.cmd)
+					self.logger.show_error('no keys are mapped to the command %r' % help_item.format_cmd())
 			if not keys:
 				continue
 			if markup:
 				markup.append(sep)
 			markup.append(ColorStr.to_markup(self.fmt, format=dict(key=self.key_sep.join(keys), cmd=help_item.name), values_are_color_str=True))
+		markup = ColorStr.simplify_markup(markup)
 		self.text.set_text(markup)
 
 	@staticmethod
