@@ -799,6 +799,32 @@ def test_save_and_load_help_item(create_test_dir: None) -> None:
 		HelpItem(CURSOR_RIGHT, 'right'),
 	]
 
+def test_save_and_load_help_item_with_alternative_commands(create_test_dir: None) -> None:
+	QUIT = ['quit', 'quit --ask', 'quit --ask-if-long-startup']
+	HELP = 'help'
+
+	class MyTestClass:
+		help_content = Config('fmt', [
+			HelpItem(QUIT, 'quit'),
+			HelpItem(HELP, 'help'),
+		])
+
+	t = MyTestClass()
+	exporter.save(FN_CONFIG)
+	assert t.help_content == [
+		HelpItem(QUIT, 'quit'),
+		HelpItem(HELP, 'help'),
+	]
+
+	t.help_content.clear()
+	assert t.help_content == []
+
+	exporter.load(FN_CONFIG)
+	assert t.help_content == [
+		HelpItem(QUIT, 'quit'),
+		HelpItem(HELP, 'help'),
+	]
+
 def test_save_and_load_command(create_test_dir: None) -> None:
 	WC_PATH = '{path}'
 	class MyTestClass:
